package exam.andexam;

import java.nio.*;

import javax.microedition.khronos.egl.*;
import javax.microedition.khronos.opengles.*;

import android.app.*;
import android.opengl.*;
import android.os.*;

public class C17_OpenGL extends Activity {
	GLSurfaceView mGLSurfaceView;
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);

		mGLSurfaceView = new GLSurfaceView(this);
		mGLSurfaceView.setRenderer(new CubeRenderer());
		setContentView(mGLSurfaceView);
	}

	protected void onResume() {
		super.onResume();
		mGLSurfaceView.onResume();
	}

	protected void onPause() {
		super.onPause();
		mGLSurfaceView.onPause();
	}
}

class CubeRenderer implements GLSurfaceView.Renderer {
	public CubeRenderer() {
		mCube = new Cube();
	}

	public void onDrawFrame(GL10 gl) {
		gl.glClear(GL10.GL_COLOR_BUFFER_BIT | GL10.GL_DEPTH_BUFFER_BIT);

		gl.glMatrixMode(GL10.GL_MODELVIEW);
		gl.glLoadIdentity();
		gl.glTranslatef(0, 0, -3.0f);
		gl.glRotatef(mAngle,        0, 1, 0);
		gl.glRotatef(mAngle*0.25f,  1, 0, 0);

		gl.glEnableClientState(GL10.GL_VERTEX_ARRAY);
		gl.glEnableClientState(GL10.GL_COLOR_ARRAY);

		mCube.draw(gl);
		mAngle += 1.2f;
	}

	public void onSurfaceChanged(GL10 gl, int width, int height) {
		gl.glViewport(0, 0, width, height);

		float ratio = (float) width / height;
		gl.glMatrixMode(GL10.GL_PROJECTION);
		gl.glLoadIdentity();
		gl.glFrustumf(-ratio, ratio, -1, 1, 1, 10);
	}

	public void onSurfaceCreated(GL10 gl, EGLConfig config) {
		gl.glDisable(GL10.GL_DITHER);
		gl.glHint(GL10.GL_PERSPECTIVE_CORRECTION_HINT, GL10.GL_FASTEST);

		gl.glClearColor(0,0,0,1);
		gl.glEnable(GL10.GL_CULL_FACE);
		gl.glShadeModel(GL10.GL_SMOOTH);
		gl.glEnable(GL10.GL_DEPTH_TEST);
	}
	private Cube mCube;
	private float mAngle;
}

class Cube
{
	public Cube()
	{
		int one = 0x10000;
		int vertices[] = { -one, -one, -one,	one, -one, -one,
							one,  one, -one,	-one,  one, -one,
							-one, -one,  one,	one, -one,  one,
							one,  one,  one,	-one,  one,  one,};
		int colors[] = {0,    0,    0,  one,	one,    0,    0,  one,
					one,  one,    0,  one,		0,  one,    0,  one,
					0,    0,  one,  one,		one,    0,  one,  one,
					one,  one,  one,  one,		0,  one,  one,  one,};
		byte indices[] = {	0, 4, 5,    0, 5, 1,	1, 5, 6,    1, 6, 2,
					2, 6, 7,    2, 7, 3,	3, 7, 4,    3, 4, 0,
					4, 7, 6,    4, 6, 5,	3, 0, 1,    3, 1, 2	};

		ByteBuffer vbb = ByteBuffer.allocateDirect(vertices.length*4);
		vbb.order(ByteOrder.nativeOrder());
		mVertexBuffer = vbb.asIntBuffer();
		mVertexBuffer.put(vertices);
		mVertexBuffer.position(0);

		ByteBuffer cbb = ByteBuffer.allocateDirect(colors.length*4);
		cbb.order(ByteOrder.nativeOrder());
		mColorBuffer = cbb.asIntBuffer();
		mColorBuffer.put(colors);
		mColorBuffer.position(0);

		mIndexBuffer = ByteBuffer.allocateDirect(indices.length);
		mIndexBuffer.put(indices);
		mIndexBuffer.position(0);
	}

	public void draw(GL10 gl)
	{
		gl.glFrontFace(GL10.GL_CW);
		gl.glVertexPointer(3, GL10.GL_FIXED, 0, mVertexBuffer);
		gl.glColorPointer(4, GL10.GL_FIXED, 0, mColorBuffer);
		gl.glDrawElements(GL10.GL_TRIANGLES, 36, GL10.GL_UNSIGNED_BYTE, mIndexBuffer);
	}

	private IntBuffer   mVertexBuffer;
	private IntBuffer   mColorBuffer;
	private ByteBuffer  mIndexBuffer;
}
